@@ -20,6 +20,7 @@
 
 from numpy import *
 from sciproc import arraysel
+from operator import itemgetter
 
 # apply a function for each element along various axis
 # data: matrix to be processed
@@ -31,47 +32,45 @@ from sciproc import arraysel
 # So the function will be applied 4x2 times. Each time, the 3 elements along second axis are
 # all passed to the function. The size of the output matrix depends on the function.
 def multifunc(data,axisref,deffunc):
-    #enige wat we eigenlijk nodig hebben voor scandata is shape(data) en shape(dataout) en dataout
-    dataout = zeros(sizeout(data,axisref,deffunc)[0])
-    SEL = list(repeat(None,len(shape(data))))
-    SELout = list(repeat(None,len(shape(dataout))))
+    # probably, we don't need this anymore
+    #dataout = zeros(sizeout(data,axisref,deffunc)[0])
+
+    reference = zip(axisref,range(len(shape(data))))
+    refsorted = sorted(reference, key=itemgetter(0,1))
+    
+    trns = []
+    trnsaxisref = []
+    for irefsorted,erefsorted in enumerate(refsorted):
+        trns.append(erefsorted[1]) 
+        trnsaxisref.append(erefsorted[0])
+    datatrns = transpose(data,trns)
+    #dataouttrns = transpose(dataout,trns)
+    print('data processing started')
+    # dataouttrns = scandata(0,datatrns,array([]),trnsaxisref,deffunc)
+    # 
+    dataouttrns = array(scandata(0,datatrns,trnsaxisref,deffunc))
+    print('data processing ended')
+    
+    #inverse permutation
+    inv = range(len(trns))
+    for itrns,etrns in enumerate(trns):
+        inv[etrns] = itrns
+    dataout = transpose(dataouttrns,inv)
+    print('shapedataout: ',shape(dataout))
 
-    # todo: dataout should be able to be of arbitrary dimension and not necessarily the size of datain!
-    scandata(0,axisref,SEL,data,deffunc,SELout,dataout)
     return dataout
 
-# this function is called by multifunc. It iteratates through the necessary dimensions to apply a function
-def scandata(dimidx,axisref,SEL,data,deffunc,SELout,dataout):
-    # we lopen elk element van een as af
-    axidx = 0
-    while (((axisref[dimidx] == True) & (axidx == 0)) | \
-            ((axisref[dimidx] == False) & (axidx < size(data,axis=dimidx)) )):
-        if (axisref[dimidx] == False):
-            SELout[dimidx] = [axidx]
-            SEL[dimidx] = [axidx]
-        else:
-            SELout[dimidx] = range(size(dataout,axis=dimidx))
-            SEL[dimidx] = range(size(data,axis=dimidx))
-        # we gaan naar een hogere dimensie
-        if (dimidx < (len(shape(data)) - 1)):
-            scandata(dimidx + 1,axisref,SEL,data,deffunc,SELout,dataout)
-        else:
-            dtempin = data[arraysel(SEL,data.shape)]
-            SHAPE = []
-            for iSHAPE in range(len(SEL)):
-                if (axisref[iSHAPE] == True):  
-                    SHAPE.append(len(SEL[iSHAPE]))
-            dtempin.shape = SHAPE
-
-            dtemp= deffunc(dtempin)
-
-            if ((type(dtemp).__name__ == 'list') or (type(dtemp).__name__ == 'tuple')):
-                dtemp = dtemp[0]
-
-            # tricky!
-            dataout[arraysel(SELout,dataout.shape)] = dtemp.ravel()
-
-        axidx = axidx + 1
+def scandata(dimidx,datain,axisref,deffunc):
+    if ((axisref[dimidx] == False) & (dimidx <= (len(shape(datain)) ))):
+        # get shape of nested dataout
+        dataout = []
+        for axidx in range(datain.shape[0]):
+            dataout.append(scandata(dimidx+1,datain[axidx],axisref,deffunc))
+    else:
+        dataout= deffunc(datain[:])
+        if ((type(dataout).__name__ == 'list') or (type(dataout).__name__ == 'tuple')):
+            dataout = dataout[0]
+    return dataout
 
 # reduce dimension of a multidimensional matrix
 # input: 
@@ -90,6 +89,7 @@ def reddim(data,indices,el = 0):
 # also called by multifunc, but may be used for other things also. 
 # ----Isn't this just doing what's done at the inner scandata loop? maybe it has to be merged with that to have one
 # single logic...---
+# probably some room for simplification!
 def sizeout(data,axisref,deffunc):
     SEL = list(repeat(None,len(shape(data))))
     sizeout = list(repeat(None,len(shape(data))))
@@ -104,7 +104,6 @@ def sizeout(data,axisref,deffunc):
     # the first item in this list contains dummy data which will not be used; 
     # the other items potentially contain the output coordinates if available
 
-    # print(shape(data[arraysel(SEL,data.shape)]))
 
     dtempin = data[arraysel(SEL,data.shape)]
     SHAPE = []
